﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using System.Device.Location;
using Windows.Devices.Geolocation;
using Microsoft.Phone.Maps.Services;
using Microsoft.Phone.Maps.Controls;
using System.Windows.Shapes;
using System.Windows.Media;
using Microsoft.Phone.Tasks;

namespace Sensors_Demos.Views
{
    public partial class MapDemos : PhoneApplicationPage
    {
        
        RouteQuery routeQuery = new RouteQuery();
        List<GeoCoordinate> waypoints;
        Geolocator geolocator;

        public MapDemos()
        {
            InitializeComponent();
            geolocator = new Geolocator();
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            routeQuery.TravelMode = TravelMode.Walking;
            routeQuery.QueryCompleted += routeQuery_QueryCompleted;
            GetCurrentLocation();
        }

        private async void GetCurrentLocation()
        {
            try
            {
                geolocator.DesiredAccuracyInMeters = 50;
                // If the cached location is over 30 minutes old, get a new one
                TimeSpan maximumAge = new TimeSpan(0, 30, 0);
                // If we're trying for more than 45 seconds, abandon the operation
                TimeSpan timeout = new TimeSpan(0, 0, 10);
                Geoposition myLocation = await geolocator.GetGeopositionAsync(maximumAge, timeout);
                GeoCoordinate geoCord = new GeoCoordinate(myLocation.Coordinate.Latitude, myLocation.Coordinate.Longitude);
                waypoints = new List<GeoCoordinate>();
                waypoints.Add(geoCord);
                MyMapControl.Center = geoCord;
                MyMapControl.ZoomLevel = 12;
                AddPoint(MyMapControl, geoCord);
                geolocator.PositionChanged += newPosition;
            }
            catch (Exception exception)
            {
                //if this is an UnauthorizedAccessException:
                //      1) you haven’t included ID_CAP_LOCATION in your app manifest
                //      or
                //      2) user has disabled location in the phone Settings

                // if this is a TimeoutExceeded exception:
                //      the device couldn't get a location in the allotted time
            }
        }

        void newPosition(Geolocator sender, PositionChangedEventArgs args)
        {
            Deployment.Current.Dispatcher.BeginInvoke(() =>
            {
                GeoCoordinate geoCord = 
                    new GeoCoordinate(args.Position.Coordinate.Latitude, 
                                        args.Position.Coordinate.Longitude);
                MyMapControl.Center = geoCord;
            });
        }

        bool isMappingRoute = false;

        private void GetGeolocationOfTap(object sender, System.Windows.Input.GestureEventArgs e)
        {
            if (isMappingRoute)
            {
                Point tapPosition = e.GetPosition((UIElement)sender);
                GeoCoordinate tapLocation = MyMapControl.ConvertViewportPointToGeoCoordinate(tapPosition);
                if (waypoints == null)
                    waypoints = new List<GeoCoordinate>();
                waypoints.Add(tapLocation);
                AddPoint(MyMapControl, tapLocation);
            }
        }

        private void AddPoint(Map controlMap, GeoCoordinate geo)
        {
            MapLayer ml = new MapLayer();
            MapOverlay mo = new MapOverlay();
            Ellipse r = new Ellipse();
            r.Width = r.Height = 12;
            r.Fill = new SolidColorBrush(Color.FromArgb(255, 240, 5, 5));
            r.Margin = new Thickness(-6, -6, 0, 0);
            mo.Content = r;
            mo.GeoCoordinate = geo;
            ml.Add(mo);
            controlMap.Layers.Add(ml);
        }

        void routeQuery_QueryCompleted(object sender, QueryCompletedEventArgs<Route> e)
        {
            if (null == e.Error)
            {
                Route MyRoute = e.Result;
                MapRoute MyMapRoute = new MapRoute(MyRoute);
                MyMapControl.AddRoute(MyMapRoute);
                MyMapControl.SetView(MyMapRoute.Route.BoundingBox);
            }
            else
                MessageBox.Show("Error occured:\n" + e.Error.Message);
        }

        private void AddRoute(object sender, System.EventArgs e)
        {
            isMappingRoute = true;
        }

        private void MappingRoute(object sender, System.EventArgs e)
        {
            isMappingRoute = false;
            routeQuery.Waypoints = waypoints;
            routeQuery.QueryAsync();
        }

        private void DownloadMapsForOffline(object sender, System.EventArgs e)
        {
            MapDownloaderTask mdt = new MapDownloaderTask();
            mdt.Show();
        }

        private void MyMapControl_ZoomLevelChanged_1(object sender, MapZoomLevelChangedEventArgs e)
        {
            MapZoomLevel.Text = MyMapControl.ZoomLevel.ToString("0.00");
        }

        
    }
}